import os
import six
import json
import random
from uuid import uuid4
from base64 import b64encode


rand_chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'


def random_string(n):
    return ''.join(random.sample(rand_chars, int(n)))


def urandom_string(n):
    value = os.urandom(int(n))
    value = b64encode(value)
    value = value.decode('utf-8')
    return value


def uuid():
    return six.text_type(uuid4())


def nacl_secretbox_key():
    from nacl import utils, secret
    value = utils.random(secret.SecretBox.KEY_SIZE)
    return b64encode(value).decode('utf-8')


def nacl_signing_key():
    from nacl import signing
    value = six.binary_type(signing.SigningKey.generate())
    return b64encode(value).decode('utf-8')


def dh_param(n=2048):
    from pyauto.pki import pkiutil
    return pkiutil.DHParam().generate(int(n)).pem_str


def rsa_key(n=2048):
    from pyauto.pki import pkiutil
    return pkiutil.RSAKey().generate(int(n)).pem_str


def traditional_rsa_key(n=2048):
    from pyauto.pki import pkiutil
    return pkiutil.RSAKey().generate(int(n)).traditional_pem_str


def pki_self_signed(profile, n, common_name, num_days):
    from pyauto.pki import pkiutil
    rsa = pkiutil.RSAKey().generate(int(n))
    csr = getattr(pkiutil.CSR(), 'create_' + profile)(common_name, rsa)
    ca = pkiutil.Cert().create_ca(common_name, int(num_days), rsa)
    crt = pkiutil.Cert().create(csr, rsa, ca, int(num_days))
    return json.dumps({
        'key': rsa.pem_str.strip(),
        'pub': rsa.public.pem_str.strip(),
        'csr': csr.pem_str.strip(),
        'ca': ca.pem_str.strip(),
        'crt': crt.pem_str.strip(),
    })


def main():
    import argparse
    from pyauto.util import funcutil
    args = argparse.ArgumentParser()
    args.add_argument('function')
    args = args.parse_args()

    function = args.function
    if args.function.startswith('.'):
        function = '__main__' + function
    func = funcutil.Function(function)
    value = func.run()
    print(value)


if '__main__' == __name__:
    main()



from . import store
from pyauto.core import config


class Secret(config.Config):
    def __init__(self, config, parent=None):
        super(Secret, self).__init__(config, parent)
        self.store = store.Store(self.filename)
        self['groups'] = [GroupDefinition(g, self) for g in self.groups]

    def get_group(self, tag):
        for g in self.groups:
            if g.get_id() == tag:
                return g
        raise Exception('unknown group: {0}'.format(tag))

    def get_value(self, group_tag, tag, namespace=None, key=None):
        value = self.get_group(group_tag).get_dict(tag)
        if namespace is None:
            return value
        if key is None:
            return value[namespace]
        return value[namespace][key]

    def delete_value(self, group_tag, tag, namespace=None, key=None):
        return self.get_group(group_tag).delete_keys(tag, namespace, key)


class GroupDefinition(config.Config):
    def __init__(self, config, parent=None):
        super(GroupDefinition, self).__init__(config, parent)
        self.definition = store.GroupDefinition(self.get_id(), self.namespaces)

    def get_group(self, tag):
        return self.config.store.get_group(self.owner_type, tag)

    def get_dict(self, tag):
        return self.get_group(tag).get_dict(self.definition.definition)

    def delete_keys(self, tag, namespace=None, key=None):
        group = self.get_group(tag)
        if namespace is not None and key is not None:
            group.delete(namespace, key)
        elif namespace is not None:
            group.delete_keys(namespace, None)
        else:
            for namespace, value in self.definition.definition.items():
                group.delete_keys(namespace, value.keys())


config.set_config_class('secret', Secret.wrap)



import os
import sys
import json
import shlex
import shutil
import importlib
from pyauto.util import yamlutil, funcutil


class Store(object):
    filename = None
    data = None

    def __init__(self, filename):
        self.filename = os.path.abspath(filename)

    def get(self, owner_type, owner, namespace, key, value_generate=None):
        self.load()
        name = '/'.join([owner_type, owner, namespace, key])
        if name not in self.data:
            if value_generate is None:
                raise Exception('value generator not specified for {0}'
                                .format(name))
            elif callable(value_generate):
                value = value_generate()
            else:
                value = value_generate
            self.data[name] = value
            self.save()
        else:
            value = self.data[name]
        return value

    def delete(self, owner_type, owner, namespace, key):
        self.load()
        name = '/'.join([owner_type, owner, namespace, key])
        if name in self.data:
            del self.data[name]
            self.save()
        return self

    def get_dict(self, owner_type, owner, values):
        self.load()
        result = {}
        for namespace, value in values.items():
            if namespace not in result:
                result[namespace] = {}
            for key, value_generate in value.items():
                result[namespace][key] = self.get(
                        owner_type, owner, namespace, key, value_generate)
        return result

    def delete_keys(self, owner_type, owner, namespace, keys):
        self.load()
        if not keys:
            name = '/'.join([owner_type, owner, namespace, ''])
            for key in [k for k in self.data.keys()]:
                if key.startswith(name):
                    del self.data[key]
        else:
            for key in keys:
                name = '/'.join([owner_type, owner, namespace, key])
                if name in self.data:
                    del self.data[name]
        self.save()
        return self

    def get_group(self, owner_type, owner):
        return Group(self, owner_type, owner)

    def save(self):
        with open(self.filename, 'w') as f:
            f.write(json.dumps(self.data, indent=2))
        return self

    def load(self):
        if os.path.isfile(self.filename):
            with open(self.filename) as f:
                self.data = json.load(f)
        else:
            self.data = {}
        return self


class Group(object):
    def __init__(self, credentials_store, owner_type, owner):
        self.owner_type = owner_type
        self.owner = owner
        self.credentials_store = credentials_store

    def get(self, namespace, key, value_generate=None):
        return self.credentials_store.get(
            self.owner_type, self.owner, namespace, key, value_generate)

    def delete(self, namespace, key):
        self.credentials_store.delete(
            self.owner_type, self.owner, namespace, key)
        return self

    def get_dict(self, values):
        return self.credentials_store.get_dict(
            self.owner_type, self.owner, values)

    def delete_keys(self, namespace, keys):
        self.credentials_store.delete_keys(
            self.owner_type, self.owner, namespace, keys)
        return self


class GroupDefinition(object):
    def __init__(self, name, definition):
        self.name = name
        self.definition = {}
        for namespace, values in definition.items():
            if not isinstance(values, dict):
                raise Exception('Invalid definitions structure. Top-level '
                                'values must be instances of dict')
            for key, value in values.items():
                func = funcutil.Function(value)

                def generate_secret(func):
                    return lambda: func.run()

                values[key] = generate_secret(func)
            self.definition[namespace] = values

    def __repr__(self):
        return str(self.definition)

    @classmethod
    def parse_definitions(cls, definitions):
        return {
            name: cls(name, definition)
            for name, definition in definitions.items()
        }

    @classmethod
    def parse_definitions_file(cls, definitions_file, definition_format):
        with open(definitions_file) as f:
            if 'json' == definition_format:
                definitions = json.load(f)
            elif 'yaml' == definition_format:
                definitions = yamlutil.load_dict(f.read())
            else:
                raise Exception('invalid format {0}'.format(definitions_format))
        return cls.parse_definitions(definitions)



import os
import sys
import json
import shlex
import shutil
import importlib
from pyauto.util import yamlutil
from .store import Store, GroupDefinition


def parse_owner(owner):
    owner, profile = owner.split(':', 1)
    owner = owner.split('/', 1)
    return [owner, profile]


def parse_key(key):
    key = key.split('/', 3)
    if len(key) not in [3, 4]:
        raise Exception('Invalid key: {0}. Cannot parse'.format(key))
    if len(key) == 3:
        key.append(None)
    else:
        key[3] = key[3].split(',')
    return key


def main():
    import argparse

    args = argparse.ArgumentParser()
    args.add_argument('-d', '--definitions-file',
                      dest='definitions_file', required=True)
    args.add_argument('-f', '--store-file',
                      dest='store_file', required=True)
    args.add_argument('-F', '--definitions-format',
                      dest='definitions_format', default='yaml',
                      choices=['yaml', 'json'], required=False)
    args.add_argument('key')
    args.add_argument('action', choices=['generate', 'delete'])
    args.add_argument('--show', dest='show', action='store_true')
    args = args.parse_args()

    store_file = os.path.abspath(args.store_file)
    definitions_file = os.path.abspath(args.definitions_file)
    group_def = GroupDefinition.parse_definitions_file(
        definitions_file, args.definitions_format)
    try:
        store = Store(store_file)
        if 'delete' == args.action:
            key = parse_key(args.key)
            store.delete_keys(*key)
        elif 'generate' == args.action:
            owner = parse_owner(args.key)
            profile = owner[1]
            owner_type, owner = owner[0]
            values = store.get_group(owner_type, owner)\
                 .get_dict(group_def[profile].definition)
        if args.show:
            with open(store_file) as f:
                shutil.copyfileobj(f, sys.stdout)
    except ModuleNotFoundError as e:
        sys.exit(str(e))


if '__main__' == __name__:
    main()



from os import path
from setuptools import setup, find_packages
from pyauto.secret import __version__


dirname = path.realpath(path.dirname(__file__))
with open(path.join(dirname, 'requirements.txt')) as f:
    reqs = [l for l in f.read().strip().split('\n') if not l.startswith('-')]

readme = path.join(dirname, 'README.md')


setup(
    name='pyauto.secret',
    version=__version__,
    description='A flexible module for generating and tracking shared secrets',
    author='Adam Jaso',
    license='GNU General Public License V2',
long_description=open(readme).read(),
    packages=find_packages(),
    install_requires=reqs,
    url='https://github.com/adamjaso/pyauto/tree/master/secret',

)



from pyauto.secret import config as secret_config
from pyauto.core import config


def secret_get(config, group, tag, namespace=None, key=None):
    return config.secret.get_value(group, tag, namespace, key)


def secret_delete(config, group, tag, namespace=None, key=None):
    return config.secret.delete_value(group, tag, namespace, key)


def secret_reset(config, group, tag, namespace=None, key=None):
    secret_delete(config, group, tag, namespace, key)
    return secret_get(config, group, tag, namespace, key)
